@@ -51,7 +51,7 @@ public:
 		m_chunked_content_parse_state(PARSE_CHUNK_SIZE_START), m_status_code(0),
 		m_bytes_content_remaining(0), m_bytes_content_read(0),
 		m_bytes_last_read(0), m_bytes_total_read(0),
-		m_max_content_length(max_content_length)
+		m_max_content_length(max_content_length), m_save_raw_headers(false)
 	{}
 
 	/// default destructor
@@ -139,6 +139,7 @@ public:
 		m_method.erase();
 		m_resource.erase();
 		m_query_string.erase();
+		m_raw_headers.erase();
 		m_bytes_content_read = m_bytes_last_read = m_bytes_total_read = 0;
 	}
 
@@ -160,6 +161,12 @@ public:
 	/// returns the maximum length for HTTP payload content
 	inline std::size_t getMaxContentLength(void) const { return m_max_content_length; }
 
+	/// returns the raw HTTP headers saved by the parser
+	inline const std::string& getRawHeaders(void) const { return m_raw_headers; }
+
+	/// returns true if the parser is saving raw HTTP header contents
+	inline bool getSaveRawHeaders(void) const { return m_save_raw_headers; }
+
 	/// returns true if the parser is being used to parse an HTTP request
 	inline bool isParsingRequest(void) const { return m_is_request; }
 
@@ -172,6 +179,9 @@ public:
 	/// resets the maximum length for HTTP payload content to the default value
 	inline void resetMaxContentLength(void) { m_max_content_length = DEFAULT_CONTENT_MAX; }
 
+	/// sets parameter for saving raw HTTP header content
+	inline void setSaveRawHeaders(bool b) { m_save_raw_headers = b; }
+
 	/// sets the logger to be used
 	inline void setLogger(PionLogger log_ptr) { m_logger = log_ptr; }
 
@@ -199,7 +209,7 @@ public:
 	 * 
 	 * @return bool true if successful
 	 */
-	static bool parseURLEncoded(HTTPTypes::StringDictionary& dict,
+	static bool parseURLEncoded(HTTPTypes::QueryParams& dict,
 								const char *ptr, const std::size_t len);
 
 	/**
@@ -209,11 +219,13 @@ public:
 	 * @param dict dictionary for key-values pairs
 	 * @param ptr points to the start of the header string to be parsed
 	 * @param len length of the encoded string, in bytes
+	 * @param set_cookie_header set true if parsing Set-Cookie response header
 	 * 
 	 * @return bool true if successful
 	 */
 	static bool parseCookieHeader(HTTPTypes::CookieParams& dict,
-								  const char *ptr, const std::size_t len);
+								  const char *ptr, const std::size_t len,
+								  bool set_cookie_header);
 
 	/**
 	 * parse key-value pairs out of a "Cookie" request header
@@ -221,13 +233,14 @@ public:
 	 * 
 	 * @param dict dictionary for key-values pairs
 	 * @param cookie_header header string to be parsed
+	 * @param set_cookie_header set true if parsing Set-Cookie response header
 	 * 
 	 * @return bool true if successful
 	 */
 	static inline bool parseCookieHeader(HTTPTypes::CookieParams& dict,
-		const std::string& cookie_header)
+		const std::string& cookie_header, bool set_cookie_header)
 	{
-		return parseCookieHeader(dict, cookie_header.c_str(), cookie_header.size());
+		return parseCookieHeader(dict, cookie_header.c_str(), cookie_header.size(), set_cookie_header);
 	}
 
 	/**
@@ -239,7 +252,7 @@ public:
 	 * 
 	 * @return bool true if successful
 	 */
-	static inline bool parseURLEncoded(HTTPTypes::StringDictionary& dict,
+	static inline bool parseURLEncoded(HTTPTypes::QueryParams& dict,
 		const std::string& query)
 	{
 		return parseURLEncoded(dict, query.c_str(), query.size());
@@ -314,12 +327,20 @@ protected:
 	 */
 	std::size_t consumeContentAsNextChunk(HTTPMessage::ChunkCache& chunk_buffers);
 
+	/**
+	 * compute and sets a HTTP Message data integrity status
+	 * @param http_msg target HTTP message 
+	 * @param msg_parsed_ok message parsing result
+	 */
+	static void computeMsgStatus(HTTPMessage& http_msg, bool msg_parsed_ok);
+
 	// misc functions used by the parsing functions
 	inline static bool isChar(int c);
 	inline static bool isControl(int c);
 	inline static bool isSpecial(int c);
 	inline static bool isDigit(int c);
 	inline static bool isHexDigit(int c);
+	inline static bool isCookieAttribute(const std::string& name, bool set_cookie_header);
 
 
 	/// maximum length for response status message
@@ -425,6 +446,9 @@ private:
 	/// Used for parsing the query string portion of a URI
 	std::string							m_query_string;
 
+	/// Used to store the raw contents of HTTP headers when m_save_raw_headers is true
+	std::string							m_raw_headers;
+
 	/// Used for parsing the name of HTTP headers
 	std::string							m_header_name;
 
@@ -454,6 +478,9 @@ private:
 
 	/// maximum length for HTTP payload content
 	std::size_t							m_max_content_length;
+	
+	/// if true, the raw contents of HTTP headers are stored into m_raw_headers
+	bool								m_save_raw_headers;
 };
 
 
@@ -492,6 +519,13 @@ inline bool HTTPParser::isHexDigit(int c)
 	return((c >= '0' && c <= '9') || (c >= 'a' && c <= 'f') || (c >= 'A' && c <= 'F'));
 }
 
+inline bool HTTPParser::isCookieAttribute(const std::string& name, bool set_cookie_header)
+{
+	return (name.empty() || name[0] == '$' || (set_cookie_header &&
+		(name=="Comment" || name=="Domain" || name=="Max-Age" || name=="Path" || name=="Secure" || name=="Version" || name=="Expires")
+		) );
+}
+
 }	// end namespace net
 }	// end namespace pion
 
