@@ -16,15 +16,6 @@
 		#pragma GCC system_header
 	#endif
 	#include <boost/asio/ssl.hpp>
-	#if defined _MSC_VER
-		#if defined _DEBUG
-			#pragma comment(lib, "ssleay32d")
-			#pragma comment(lib, "libeay32d")
-		#else
-			#pragma comment(lib, "ssleay32")
-			#pragma comment(lib, "libeay32")
-		#endif
-	#endif 
 #endif
 
 #include <boost/noncopyable.hpp>
@@ -596,7 +587,7 @@ public:
 	 */
 	template <typename ConstBufferSequence>
 	inline std::size_t write(const ConstBufferSequence& buffers,
-							 boost::system::error_code ec)
+							 boost::system::error_code& ec)
 	{
 #ifdef PION_HAVE_SSL
 		if (getSSLFlag())
@@ -680,6 +671,23 @@ public:
 		return getRemoteEndpoint().port();
 	}
 	
+	/// returns reference to the io_service used for async operations
+	inline boost::asio::io_service& getIOService(void) {
+		return m_tcp_socket.io_service();
+	}
+
+	/// returns non-const reference to underlying TCP socket object
+	inline Socket& getSocket(void) { return m_tcp_socket; }
+	
+	/// returns non-const reference to underlying SSL socket object
+	inline SSLSocket& getSSLSocket(void) { return m_ssl_socket; }
+
+	/// returns const reference to underlying TCP socket object
+	inline const Socket& getSocket(void) const { return m_tcp_socket; }
+	
+	/// returns const reference to underlying SSL socket object
+	inline const SSLSocket& getSSLSocket(void) const { return m_ssl_socket; }
+
 	
 protected:
 		
