@@ -43,6 +43,10 @@ boost::tribool HTTPParser::parse(HTTPMessage& http_msg)
 	boost::tribool rc = boost::indeterminate;
 	std::size_t total_bytes_parsed = 0;
 
+	if(http_msg.hasMissingPackets()) {
+		http_msg.setDataAfterMissingPacket(true);
+	}
+
 	do {
 		switch (m_message_parse_state) {
 			// just started parsing the HTTP message
@@ -94,6 +98,8 @@ boost::tribool HTTPParser::parse(HTTPMessage& http_msg)
 	if (rc == true) {
 		m_message_parse_state = PARSE_END;
 		finish(http_msg);
+	} else if(rc == false) {
+		computeMsgStatus(http_msg, false);
 	}
 
 	// update bytes last read (aggregate individual operations for caller)
@@ -107,6 +113,8 @@ boost::tribool HTTPParser::parseMissingData(HTTPMessage& http_msg, std::size_t l
 	static const char MISSING_DATA_CHAR = 'X';
 	boost::tribool rc = boost::indeterminate;
 
+	http_msg.setMissingPackets(true);
+
 	switch (m_message_parse_state) {
 
 		// cannot recover from missing data while parsing HTTP headers
@@ -189,6 +197,8 @@ boost::tribool HTTPParser::parseMissingData(HTTPMessage& http_msg, std::size_t l
 	if (rc == true) {
 		m_message_parse_state = PARSE_END;
 		finish(http_msg);
+	} else if(rc == false) {
+		computeMsgStatus(http_msg, false);
 	}
 
 	return rc;
@@ -520,7 +530,10 @@ boost::tribool HTTPParser::parseHeaders(HTTPMessage& http_msg)
 			m_bytes_total_read += m_bytes_last_read;
 			return true;
 		}
-
+		
+		if (m_save_raw_headers)
+			m_raw_headers += *m_read_ptr;
+		
 		++m_read_ptr;
 	}
 
@@ -549,7 +562,7 @@ void HTTPParser::updateMessageWithHeaderData(HTTPMessage& http_msg) const
 					<< m_query_string << "\"");
 		}
 
-		// parse "Cookie" headers
+		// parse "Cookie" headers in request
 		std::pair<HTTPTypes::Headers::const_iterator, HTTPTypes::Headers::const_iterator>
 		cookie_pair = http_request.getHeaders().equal_range(HTTPTypes::HEADER_COOKIE);
 		for (HTTPTypes::Headers::const_iterator cookie_iterator = cookie_pair.first;
@@ -557,7 +570,7 @@ void HTTPParser::updateMessageWithHeaderData(HTTPMessage& http_msg) const
 			 && cookie_iterator != cookie_pair.second; ++cookie_iterator)
 		{
 			if (! parseCookieHeader(http_request.getCookieParams(),
-									cookie_iterator->second) )
+									cookie_iterator->second, false) )
 				PION_LOG_WARN(m_logger, "Cookie header parsing failed");
 		}
 
@@ -568,6 +581,19 @@ void HTTPParser::updateMessageWithHeaderData(HTTPMessage& http_msg) const
 		HTTPResponse& http_response(dynamic_cast<HTTPResponse&>(http_msg));
 		http_response.setStatusCode(m_status_code);
 		http_response.setStatusMessage(m_status_message);
+
+		// parse "Set-Cookie" headers in response
+		std::pair<HTTPTypes::Headers::const_iterator, HTTPTypes::Headers::const_iterator>
+		cookie_pair = http_response.getHeaders().equal_range(HTTPTypes::HEADER_SET_COOKIE);
+		for (HTTPTypes::Headers::const_iterator cookie_iterator = cookie_pair.first;
+			 cookie_iterator != http_response.getHeaders().end()
+			 && cookie_iterator != cookie_pair.second; ++cookie_iterator)
+		{
+			if (! parseCookieHeader(http_response.getCookieParams(),
+									cookie_iterator->second, true) )
+				PION_LOG_WARN(m_logger, "Set-Cookie header parsing failed");
+		}
+
 	}
 }
 
@@ -641,7 +667,7 @@ boost::tribool HTTPParser::finishHeaderParsing(HTTPMessage& http_msg)
 	return rc;
 }
 
-bool HTTPParser::parseURLEncoded(HTTPTypes::StringDictionary& dict,
+bool HTTPParser::parseURLEncoded(HTTPTypes::QueryParams& dict,
 								 const char *ptr, const size_t len)
 {
 	// used to track whether we are parsing the name or value
@@ -713,9 +739,11 @@ bool HTTPParser::parseURLEncoded(HTTPTypes::StringDictionary& dict,
 }
 
 bool HTTPParser::parseCookieHeader(HTTPTypes::CookieParams& dict,
-								   const char *ptr, const size_t len)
+								   const char *ptr, const size_t len,
+								   bool set_cookie_header)
 {
 	// BASED ON RFC 2109
+	// http://www.ietf.org/rfc/rfc2109.txt
 	// 
 	// The current implementation ignores cookie attributes which begin with '$'
 	// (i.e. $Path=/, $Domain=, etc.)
@@ -747,7 +775,7 @@ bool HTTPParser::parseCookieHeader(HTTPTypes::CookieParams& dict,
 				// when quoted values are encountered
 				if (! cookie_name.empty()) {
 					// value is empty (OK)
-					if (cookie_name[0] != '$')
+					if (! isCookieAttribute(cookie_name, set_cookie_header))
 						dict.insert( std::make_pair(cookie_name, cookie_value) );
 					cookie_name.erase();
 				}
@@ -766,7 +794,7 @@ bool HTTPParser::parseCookieHeader(HTTPTypes::CookieParams& dict,
 				// value is not (yet) quoted
 				if (*ptr == ';' || *ptr == ',') {
 					// end of value found (OK if empty)
-					if (cookie_name[0] != '$') 
+					if (! isCookieAttribute(cookie_name, set_cookie_header))
 						dict.insert( std::make_pair(cookie_name, cookie_value) );
 					cookie_name.erase();
 					cookie_value.erase();
@@ -793,7 +821,7 @@ bool HTTPParser::parseCookieHeader(HTTPTypes::CookieParams& dict,
 				// value is quoted
 				if (*ptr == value_quote_character) {
 					// end of value found (OK if empty)
-					if (cookie_name[0] != '$') 
+					if (! isCookieAttribute(cookie_name, set_cookie_header))
 						dict.insert( std::make_pair(cookie_name, cookie_value) );
 					cookie_name.erase();
 					cookie_value.erase();
@@ -819,7 +847,7 @@ bool HTTPParser::parseCookieHeader(HTTPTypes::CookieParams& dict,
 	}
 
 	// handle last cookie in string
-	if (! cookie_name.empty() && cookie_name[0] != '$')
+	if (! isCookieAttribute(cookie_name, set_cookie_header))
 		dict.insert( std::make_pair(cookie_name, cookie_value) );
 
 	return true;
@@ -1046,6 +1074,8 @@ void HTTPParser::finish(HTTPMessage& http_msg) const
 		break;
 	}
 
+	computeMsgStatus(http_msg, http_msg.isValid());
+
 	if (isParsingRequest()) {
 		// Parse query pairs from post content if content type is x-www-form-urlencoded.
 		// Type could be followed by parameters (as defined in section 3.6 of RFC 2616)
@@ -1064,6 +1094,20 @@ void HTTPParser::finish(HTTPMessage& http_msg) const
 	}
 }
 
+void HTTPParser::computeMsgStatus(HTTPMessage& http_msg, bool msg_parsed_ok )
+{
+	HTTPMessage::DataStatus st = HTTPMessage::STATUS_NONE;
+
+	if(http_msg.hasMissingPackets()) {
+		st = http_msg.hasDataAfterMissingPackets() ?
+						HTTPMessage::STATUS_PARTIAL : HTTPMessage::STATUS_TRUNCATED;
+		http_msg.setStatus(st);
+	} else {
+		st = msg_parsed_ok ? HTTPMessage::STATUS_OK : HTTPMessage::STATUS_TRUNCATED;
+	}
+
+	http_msg.setStatus(st);
+}
 }	// end namespace net
 }	// end namespace pion
 
