@@ -44,12 +44,12 @@ PION_DECLARE_PLUGIN(CookieService)
 #if defined(PION_XCODE)
 	static const std::string PATH_TO_PLUGINS(".");
 	static const std::string SSL_PEM_FILE("../../net/utils/sslkey.pem");
-	static const std::string SERVICES_CONFIG_FILE("../../net/utils/testservices.conf");
+	static const std::string SERVICES_CONFIG_FILE("../../net/tests/config/testservices.conf");
 #else
 	// same for Unix and Windows
 	static const std::string PATH_TO_PLUGINS("../services/.libs");
 	static const std::string SSL_PEM_FILE("../utils/sslkey.pem");
-	static const std::string SERVICES_CONFIG_FILE("../utils/testservices.conf");
+	static const std::string SERVICES_CONFIG_FILE("../tests/config/testservices.conf");
 #endif
 
 
@@ -650,12 +650,6 @@ BOOST_AUTO_TEST_CASE(checkAllowNothingServiceResponseContent) {
 #endif // PION_STATIC_LINKING
 
 BOOST_AUTO_TEST_CASE(checkFileServiceResponseContent) {
-#ifdef PION_XCODE
-	BOOST_REQUIRE_MESSAGE(boost::filesystem::exists("../../net/doc/html"), "running Doxygen should fix this");
-#else
-	BOOST_REQUIRE_MESSAGE(boost::filesystem::exists("../doc/html"), "running Doxygen should fix this");
-#endif
-
 	// load multiple services and start the server
 	m_server.loadServiceConfig(SERVICES_CONFIG_FILE);
 	m_server.start();
@@ -669,7 +663,7 @@ BOOST_AUTO_TEST_CASE(checkFileServiceResponseContent) {
 	checkWebServerResponseContent(http_stream, "/" , index_page_regex);
 	checkWebServerResponseContent(http_stream, "/index.html" , index_page_regex);
 
-	// send request and check response (docs index page: requires net/doc/html doxygen files!)
+	// send request and check response (copy of docs index page generated by doxygen)
 	const boost::regex doc_index_regex(".*<html>.*pion-.*Documentation.*</html>.*");
 	checkWebServerResponseContent(http_stream, "/doc/index.html" , doc_index_regex);
 }
